/*

MyL grammar

version 02		04-Aug-2008

*/

grammar MyL; 
options {
    language=CSharp;
	ASTLabelType=CommonTree;
    output = AST;
}


tokens {
    PLUS;
    MINUS;
    MULTIPLY;
    DIVIDE;
    UNARY_PLUS;
    UNARY_MINUS;
}

@namespace { MyL.Parser.AST }

@parser::header {
    // turn of unreachable code warnings
    // turn off obsolete code warnings
    // turn off unused variable
    # pragma warning disable 0162,0618,0219 
	using System.Text;
}

@lexer::header {
    // turn of unreachable code warnings
    // turn off obsolete code warnings
    // turn off unused variable
    # pragma warning disable 0162,0618,0219 
	using System.Text;
}

//**** PARSER ****//
	
program		: expr NEWLINE! EOF!
			;
	
expr		: multExpr ((PLUS|MINUS)^ multExpr)*
			| IDENTIFIER ASSIGN expr -> ^(ASSIGN IDENTIFIER expr)
			;
	
multExpr	: unaryExpr ((MULTIPLY|DIVIDE)^ unaryExpr)*
			;

unaryExpr	: PLUS primary -> ^(UNARY_PLUS primary )
			| MINUS primary -> ^(UNARY_MINUS primary )
			| primary
			;
	
primary		: NUMBER
			| IDENTIFIER
			| LPAREN! expr RPAREN!
			;

//**** LEXER ****//

PLUS		: '+';
	
MINUS		: '-';
	
MULTIPLY	: '*';
	
DIVIDE		: '/';

LPAREN		: '(';
	
RPAREN		: ')'; 

ASSIGN		: '=' ;

WS			: (' ' | '\t')+ {Skip();};

NEWLINE		: '\r'? '\n';

IDENTIFIER	: LCLETTER (LCLETTER | DIGIT)*;

NUMBER		: INT | FLOAT;

fragment
FLOAT		: (DIGIT)+ '.' (DIGIT)+;
 
fragment	
INT			: (DIGIT)+;
		
fragment
DIGIT		: '0'..'9';

fragment
LCLETTER	: 'a'..'z';