/*

MyL tree grammar

version 02		04-Aug-2008
	
*/

tree grammar MyLTree;

options {
	tokenVocab=MyL;
    language = CSharp;
	ASTLabelType = CommonTree;
	output=AST;
}

@namespace { MyL.Parser.AST }

@header {   
    // turn of unreachable code warnings
    // turn off obsolete code warnings
    // turn off unused variable
    # pragma warning disable 0162,0618,0219 
	using Microsoft.Scripting;
}


//**** WALKER ****//

program returns [MyLStatement result]
	: e=expr  {$result = new MyLPrint(new SourceSpan(), $e.result);}
	| ^(ASSIGN lvalue=IDENTIFIER rvalue=expr) {
			MyLIdentifier id;
			MyLAssign assign;
			
			id = new MyLIdentifier(new SourceSpan(), $lvalue.Text);
			assign = new MyLAssign(new SourceSpan(), id, $rvalue.result);
			$result = new MyLPrint(new SourceSpan(), assign);
			}
	;

expr returns [MyLExpression result]
	: ^(op=(PLUS|MINUS|MULTIPLY|DIVIDE) left=expr right=expr) {$result = new MyLBinary(new SourceSpan(), $op.Token.Type, $left.result, $right.result);}
	| ^(op=(UNARY_PLUS|UNARY_MINUS) arg=expr) {$result = new MyLUnary(new SourceSpan(), $op.Token.Type, $arg.result);}
	| id=IDENTIFIER {$result = new MyLIdentifier(new SourceSpan(), $id.Text);}
	| n=NUMBER {$result = new MyLConstant(new SourceSpan(), double.Parse($n.Text));}
	;